/*
  This program transforms two DLV style programs to two first-order theories.
  If both are unsatisfiable, the programs are strongly equivalent.
  Copyright (C) 2004 Patrick Traxler

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

using namespace std;
#include<string>
#define YYSTYPE string*

#include<iostream>
#include<stdlib.h>
#include <list>

#include <fstream>
//#include <string.h>
#include <stdio.h>

class SymbolTable {
private:
    struct Symbol {
        string s;
        int arity;
        Symbol(const string& u, int a = 0) {
            s     = u;
            arity = a;
        }
        Symbol& operator=(const Symbol& sym) {
            s     = sym.s;
            arity = sym.arity;
            return *this;
        }
        bool operator==(const Symbol& sym) {
            return s == sym.s;
        }
        bool operator!=(const Symbol& sym) {
            return !(*this == sym);
        }
    };
    list<Symbol> l;
    list<Symbol>::const_iterator itr;
    int next_symbol;

public:
    SymbolTable() {
        next_symbol = 1;
    }

    void   add(const string& , int arity = 0);
    bool   exists(const string& s);
    int    getArity(const string& s);
    string newSymbol(const string& prefix, int arity = 0);

    string iterate(int);

    int    size() {
        return l.size();
    }
    void   print();
};

extern SymbolTable st_CS, st_RS;



class Rule {
private:
    string s;

private:
    void substitute(const string& t, const string& u);
    void clean();

public:
    bool nextRule(string& s);

    void skolemize();
    string nextLiteral();

    string getStr() const {
        return s;
    }
    Rule& operator=(const Rule& r) {
        s = r.getStr();
        return *this;
    }
};
